rem GNU Solfege - free eartraining
rem Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
rem
rem This file is installed into c:\Program files\GNU Solfege and is called
rem from the start menu to run the program

set HOME=\home
solfege.exe

rem If you want to run from the source directory without creating the
rem installer and installing, you must comment out the two lines above
rem and uncomment (and most likely modify paths) the lines below.

rem PATH=c:\python21
rem set HOME=\home
rem set PYTHONPATH=c:\winport\pygtk
rem python solfege.py
