rem GNU Solfege - free eartraining
rem Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
rem
rem This file is installed into c:\Program files\GNU Solfege and is called
rem from the start menu when you select 'Solfege (with console)
rem
rem The purpose if two ways to run Solfege is that selecting 'with console'
rem a console window is visible, and it is pause if the program crash, so
rem you can see error messages.

set HOME=\home
solfege.exe
pause
