# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
You have to call initialise to get translated messages.
"""

import gettext

def _(s):
    return s

def _i(s):
    return s

def initialise(localedir):
    global _, _i
    gettext.bindtextdomain("solfege", localedir)
    gettext.textdomain("solfege")
    _ = gettext.gettext

    cat = gettext.Catalog("solfege-intervallnames", localedir)
    _i = cat.gettext
