#!/usr/bin/env python

# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without
# modifications, as long as this notice is preserved.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
"""
I used this to display the saved statistics in $HOME/.solfegeX.Y/
"""
import pickle, sys
for fn in sys.argv[1:]:
    f = open(fn, "r")
    D = pickle.load(f)
    print "Un-pickled file:", fn
    print D
    f.close()
