#!/usr/bin/python

# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, sys, string

RM = "tmp/tmpsolfege"
prefix = '/usr'

sys.stdout.write("%defattr(-,root,root)\n")
infile = sys.stdin
while 1:
    s = infile.readline()
    if not s:
        break
    s = '/'+s[:-1][len(RM)+2:]
    if s: #UGH WHY WHY WHY!?!
        if s == '/usr/man/man1/solfege.1':
            s = s + "*"
        sys.stdout.write(s)
        sys.stdout.write("\n")

