#!/usr/bin/env python

import profile, os
import types, sys

sys.path.insert(0, "gnomeemu")
sys.path.append(".")
sys.path.append("/usr/lib/python1.5/site-packages")

import src.lessonfile

def par():
	for n in range(100):
		src.lessonfile.DictationLessonfile(sys.argv[1])

import time
print "Startet tidtaking:"
t1 = time.clock()
par()
t2 = time.clock()
print t2-t1
#profile.run("par()", "profile.txt")

#import pstats
#p = pstats.Stats("profile.txt")
##p.strip_dirs().sort_stats('time').print_stats(30)
#p.strip_dirs().sort_stats('cumulative').print_stats(30)

#bas()
